#  Copyright (c) 2015-2018 Cisco Systems, Inc.
#  Copyright (c) 2018 Red Hat, Inc.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
"""Functional Tests."""
import os

from molecule import logger
from molecule.test.conftest import change_dir_to, molecule_directory
from molecule.util import run_command

LOG = logger.get_logger(__name__)


def test_command_init_scenario(temp_dir):
    """Verify that we can initialize a new scenario with this driver."""
    with change_dir_to(temp_dir):
        scenario_directory = os.path.join(molecule_directory(), "default")
        cmd = [
            "molecule",
            "init",
            "scenario",
            "default",
            "--driver-name",
            "containers",
        ]
        result = run_command(cmd)
        assert result.returncode == 0

        assert os.path.isdir(scenario_directory)

        # we do not run the full "test" sequence because lint will fail, check
        # is shorter but comprehensive enough to test the most important
        # functionality: destroy, dependency, create, prepare, converge
        cmd = ["molecule", "check", "-s", "default"]
        result = run_command(cmd)
        assert result.returncode == 0



"""Containers Driver Module."""
from __future__ import absolute_import

import inspect
import os
import shutil

from molecule import logger

_logger = logger.get_logger(__name__)

# Preference logic for picking backend driver to be used.
drivers = os.environ.get("MOLECULE_CONTAINERS_BACKEND", "podman,docker").split(",")
for driver in drivers:
    if shutil.which(driver):
        break
else:
    driver = drivers[0]

# Logic for picking backend is subject to change.
if driver == "docker":
    from molecule_docker.driver import Docker as DriverBackend
elif driver == "podman":
    from molecule_podman.driver import Podman as DriverBackend
else:
    raise NotImplementedError(f"Driver {driver} is not supported.")
_logger.debug("Containers driver will use %s backend", driver)


class Container(DriverBackend):
    """
    Container Driver Class.

    This class aims to provide an agnostic container enginer implementation,
    which should allow users to consume whichever enginer they have available.
    """  # noqa

    def __init__(self, config=None):
        """Construct Container."""
        super().__init__(config)
        self._name = "containers"
        # Assure that _path points to the driver we would be using, or
        # molecule will fail to find the embedded playbooks.
        self._path = os.path.abspath(os.path.dirname(inspect.getfile(DriverBackend)))
